<?php

    class Create_Nagiosim_Incident_Action extends Reactor_Action
    {
        protected $tpl = 'create_nagiosim_incident.php';
        protected $name = 'Create a new Nagios IM Incident';
		
		public static $registration = array(
			ACTION_NAME => 'Create a new Nagios IM Incident',
			ACTION_CATEGORY => 'Ticketing',
			ACTION_DESCRIPTION => 'Creates a new incident with Nagios Incident Manager.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 		
		
        protected $fields = array(
            'api_url' => array(
                'label' => 'api url',
                'validate' => 'trim|required|xss_clean|callback_check_url',
            ),
            'api_key' => array(
                'label' => 'api key',
                'validate' => 'trim|required|xss_clean|callback_check_api_key',
            ),
            'title' => array(
                'label' => 'title',
                'validate' => 'trim|required',
            ),
            'summary' => array(
                'label' => 'summary',
                'validate' => 'trim|required',
            ),
            'type' => array(
                'label' => 'type',
                'validate' => 'trim|required',
            ),
            'priority' => array(
                'label' => 'priority',
                'validate' => 'trim'
            ),
            'status' => array(
                'label' => 'status',
                'validate' => 'trim'
            ),
            'users' => array(
                'label' => 'users',
                'validate' => 'trim'
            ),
            'teams' => array(
                'label' => 'teams',
                'validate' => 'trim'
            ),
        );

        /** 
         * Form validation function to check a url
         **/
        public function check_url($str)
        {
            $url_expression = "/^(https?|ftp)\:\/\/([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?[a-z0-9+\$_-]+(\.[a-z0-9+\$_-]+)*(\:[0-9]{2,5})?(\/([a-z0-9+\$_-]\.?)+)*\/?(\?[a-z+&\$_.-][a-z0-9;:@\/&%=+\$_.-]*)?(#[a-z_.-][a-z0-9+\$_.-]*)?\$/i";
            if(!preg_match($url_expression, $str)) {
                $this->ci->form_validation->set_message('check_url', 'The url specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            if(!in_array('curl', get_loaded_extensions())) {
                $this->log('error', 'cURL is not installed and is required for this function.');
                $this->set_result(FALSE);
                return;
            }
            $api_url = sprintf("%s/incidents/add", $this->get_param('api_url'));
            $post_query = array();
            $post_query_str = "";
            $post_query['api_key'] = $this->get_param('api_key');
            $post_query['title'] = $this->get_param('title');
            $post_query['summary'] = $this->get_param('summary');
            $post_query['type'] = $this->get_param('type');
            if($priority = $this->get_param('priority')) {
                $post_query['priority'] = $priority;
            }
            if($status = $this->get_param('status')) {
                $post_query['status'] = $status;
            }
            foreach($post_query as $name => $val) {
                $post_query_str .= sprintf("%s=%s&", $name, urlencode($val));
            }

            if($users = $this->get_param('users')) {
                $user_arr = explode("\n", $users);
                $i=0;
                foreach($user_arr as $user) {
                    if(strlen($user)) {
                        $post_query_str .= sprintf("users[%d]=%s&", $i++, trim(urlencode($user)));
                    }
                }
            }
                        
            if($teams = $this->get_param('teams')) {
                $team_arr = explode("\n", $teams);
                $i=0;
                foreach($team_arr as $team) {
                    if(strlen($team)) {
                        $post_query_str .= sprintf("teams[%d]=%s&", $i++, trim(urlencode($team)));
                    }
                }
            }
                        
            $post_query_str = rtrim($post_query_str, "&");

            $this->log('info', sprintf('Performing POST request to %s', $api_url));
            list($http_result, $http_info) = do_curl_request($api_url, $post_query_str);
            $this->log('info', 'HTTP status was '.$http_info['http_code']);
            $s = $http_info['http_code'];
            $this->set_result(($s >= 200 && $s < 300) ? 0 : 1);
            return $context;
        }
    }
